@@ -4,7 +4,7 @@
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
  
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+    along with this program; if not, see <http://www.gnu.org/licenses/>.
  
     */
 
@@ -547,7 +546,7 @@ parse_reg_property(device *current,
   device_add_reg_array_property(current, property_name,
 				regs, nr_regs);
 
-  zfree(regs);
+  free(regs);
 }
 
 
@@ -584,7 +583,7 @@ parse_ranges_property(device *current,
   /* create it */
   device_add_range_array_property(current, property_name, ranges, nr_ranges);
 
-  zfree(ranges);
+  free(ranges);
 }
 
 
@@ -770,9 +769,9 @@ parse_string_property(device *current,
   /* flush the created string */
   while (nr_strings > 0) {
     nr_strings--;
-    zfree(strings[nr_strings]);
+    free(strings[nr_strings]);
   }
-  zfree(strings);
+  free(strings);
 }
 
 
